angular.module("fng.ui.bootstrap", ["fng.ui.bootstrap.tpls", "fng.ui.bootstrap.dropdownToggle","fng.ui.bootstrap.tabs"]);
angular.module("fng.ui.bootstrap.tpls", ["template/tabs/pane.html","template/tabs/tabs.html"]);
/*
 * dropdownToggle - Provides dropdown menu functionality in place of bootstrap js
 * @restrict class or attribute
 * @example:
 <li class="dropdown">
 <a class="dropdown-toggle">My Dropdown Menu</a>
 <ul class="dropdown-menu">
 <li ng-repeat="choice in dropChoices">
 <a ng-href="{{choice.href}}">{{choice.text}}</a>
 </li>
 </ul>
 </li>
 */

angular.module('fng.ui.bootstrap.dropdownToggle', []).directive('dropdownToggle',
    ['$document', '$location', '$window', function ($document, $location, $window) {
        var openElement = null,
            closeMenu   = angular.noop;
        return {
            restrict: 'CA',
            link: function(scope, element, attrs) {
                scope.$watch('$location.path', function() { closeMenu(); });
                element.parent().bind('click', function() { closeMenu(); });
                element.bind('click', function(event) {
                    event.preventDefault();
                    event.stopPropagation();
                    var elementWasOpen = (element === openElement);
                    if (!!openElement) {
                        closeMenu(); }
                    if (!elementWasOpen){
                        element.parent().addClass('open');
                        openElement = element;
                        closeMenu = function (event) {
                            if (event) {
                                event.preventDefault();
                                event.stopPropagation();
                            }
                            $document.unbind('click', closeMenu);
                            element.parent().removeClass('open');
                            closeMenu   = angular.noop;
                            openElement = null;
                        };
                        $document.bind('click', closeMenu);
                    }
                });
            }
        };
    }]);
angular.module('fng.ui.bootstrap.tabs', [])
    .controller('TabsController', ['$scope', '$element', function($scope, $element) {
        var panes = $scope.panes = [];

        this.select = $scope.select = function selectPane(pane) {
            angular.forEach(panes, function(pane) {
                pane.selected = false;
            });
            pane.selected = true;
        };

        this.addPane = function addPane(pane) {
            if (!panes.length) {
                $scope.select(pane);
            }
            panes.push(pane);
        };

        this.removePane = function removePane(pane) {
            var index = panes.indexOf(pane);
            panes.splice(index, 1);
            //Select a new pane if removed pane was selected
            if (pane.selected && panes.length > 0) {
                $scope.select(panes[index < panes.length ? index : index-1]);
            }
        };
    }])
    .directive('tabs', function() {
        return {
            restrict: 'EA',
            transclude: true,
            scope: {},
            controller: 'TabsController',
            templateUrl: 'template/tabs/tabs.html',
            replace: true
        };
    })
    .directive('pane', ['$parse', function($parse) {
        return {
            require: '^tabs',
            restrict: 'EA',
            transclude: true,
            scope:{
                heading:'@'
            },
            link: function(scope, element, attrs, tabsCtrl) {
                var getSelected, setSelected;
                scope.selected = false;
                if (attrs.active) {
                    getSelected = $parse(attrs.active);
                    setSelected = getSelected.assign;
                    scope.$watch(
                        function watchSelected() {return getSelected(scope.$parent);},
                        function updateSelected(value) {scope.selected = value;}
                    );
                    scope.selected = getSelected ? getSelected(scope.$parent) : false;
                }
                scope.$watch('selected', function(selected) {
                    if(selected) {
                        tabsCtrl.select(scope);
                    }
                    if(setSelected) {
                        setSelected(scope.$parent, selected);
                    }
                });

                tabsCtrl.addPane(scope);
                scope.$on('$destroy', function() {
                    tabsCtrl.removePane(scope);
                });
            },
            templateUrl: 'template/tabs/pane.html',
            replace: true
        };
    }]);

angular.module("template/tabs/pane.html", []).run(["$templateCache", function($templateCache){
    $templateCache.put("template/tabs/pane.html",
        "<div class=\"tab-pane\" ng-class=\"{active: selected}\" ng-show=\"selected\" ng-transclude></div>" +
            "");
}]);

angular.module("template/tabs/tabs.html", []).run(["$templateCache", function($templateCache){
    $templateCache.put("template/tabs/tabs.html",
        "<div class=\"tabbable\">" +
            "  <ul class=\"nav nav-tabs\">" +
            "    <li ng-repeat=\"pane in panes\" ng-class=\"{active:pane.selected}\">" +
            "      <a ng-click=\"select(pane)\">{{pane.heading}}</a>" +
            "    </li>" +
            "  </ul>" +
            "  <div class=\"tab-content\" ng-transclude></div>" +
            "</div>" +
            "");
}]);
